"""Basic layout components.
"""
import grok
from hurry import yui
from megrok.layout import Page, Layout, EditForm, DisplayForm, AddForm
from waeup.interfaces import IWAeUPObject
from z3c.flashmessage.interfaces import IMessageSource
from zope.component import queryUtility, ComponentLookupError
from zope.interface import Interface
# Conditional import. getSite moved to zope.site.hooks in recent grok versions.
try:
    from zope.site.hooks import getSite
except ImportError:
    from zope.app.component.hooks import getSite

grok.templatedir('templates')

def NullValidator(*args, **kw):
    """A validator that does not validate.

    This is needed especially for cancel buttons. We don't want data
    to be validated that will be thrown away in the next step.

    You can use it with ``grok.action`` decorator like this::

      @grok.action('Cancel', validator=NullValidator)
      def cancel(self, **data):
        self.redirect(<whereever-you-go>)
    """
    return dict()

class UtilityView(object):
    """A view mixin with useful methods.

    The ``pnav`` attribute (a number) tells, to which primary
    navigation tab a page declares to belong.
    """
    title = u'' # What appears in the content title...
    pnav = 0 # Primary navigation index...
    
    def application_url(self, name=None):
        """Return the URL of the nearest site.
        """
        site = getSite()
        #if not site:
        #    raise ComponentLookupError("No site found.")
        return self.url(site, name)

    def flash(self, message, type='message'):
        """Send a short message to the user.
        """
        source = queryUtility(IMessageSource, name='session')
        if source is None:
            return None
        source.send(message, type)
        return True

class WAeUPLayout(Layout, UtilityView):
    """A megrok.layout.Layout with additional methods.
    """
    grok.baseclass()
    
class WAeUPPage(Page, UtilityView):
    """A megrok.layout page with additional methods.
    """
    grok.baseclass()

class WAeUPDisplayFormPage(DisplayForm, UtilityView):
    """A megrok.layout.DisplayForm with additional methods.
    """
    grok.baseclass()

class WAeUPEditFormPage(EditForm, UtilityView):
    """A megrok.layout.EditForm with additional methods.
    """
    grok.baseclass()

class WAeUPAddFormPage(AddForm,  UtilityView):
    """A megrok.layout.AddForm with additional methods.
    """
    grok.baseclass()

class SiteLayout(WAeUPLayout):
    """ The general site layout.
    """
    grok.context(IWAeUPObject)

    def getAppTitle(self):
        return getattr(grok.getSite(), 'name', u'Sample University')
    
    def isAuthenticated(self):
        """Return True if the calling user is authenticated.
        """
        usertitle = self.request.principal.title
        return usertitle != 'Unauthenticated User'
    
    def getUserTitle(self):
        """Return principal title of current user.
        """
        usertitle = self.request.principal.title
        if usertitle == 'Unauthenticated User':
            return u''
        return usertitle

    def update(self):
        yui.reset_fonts_grids.need()
        yui.sam.need()
